/*****************************************************************************
 *
 *   Copyright(C) 2012, Embedded Artists AB
 *   All rights reserved.
 *
 *   This project is based on Code Red Technologies systick example
 *
 ******************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * Embedded Artists AB assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. Embedded Artists AB
 * reserves the right to make changes in the software without
 * notification. Embedded Artists AB also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
 *****************************************************************************/

/******************************************************************************
 * Includes
 *****************************************************************************/

#ifdef __USE_CMSIS
#include "LPC11Uxx.h"
#endif

#include <cr_section_macros.h>
#include <NXP/crp.h>

#include "gpio.h"

/******************************************************************************
 * Typedef and defines
 *****************************************************************************/

// p0.7 is connected to LED3 on LPC11U35 QSB Board
#define LED_PORT 0  // Port for led
#define LED_BIT  7  // Bit on port for led
#define LED_ON   1  // Level to set port to turn on led
#define LED_OFF  0  // Level to set port to turn off led

// delay when toggling the LED
#define DELAY_LEN_MS 500

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;


/******************************************************************************
 * Local variables
 *****************************************************************************/

static volatile uint32_t msTicks;

/******************************************************************************
 * Local functions
 *****************************************************************************/

/******************************************************************************
 *
 * Description:
 *    Delay function
 *
 * Params:
 *    ms - number of milliseconds to wait
 *
 *****************************************************************************/
__INLINE static void delay_ms (uint32_t ms) {
  uint32_t curTicks;

  curTicks = msTicks;
  while ((msTicks - curTicks) < ms);
}

/******************************************************************************
 * Public functions
 *****************************************************************************/

void SysTick_Handler(void) {
  msTicks++;
}

/******************************************************************************
 *
 * Description:
 *    Main function
 *
 *****************************************************************************/
int main(void) {

  // Initialize GPIO
  GPIOInit();

  // Set LED pin as output
  GPIOSetDir(LED_PORT, LED_BIT, 1 );

  // enable 1 ms timer tick
  if (SysTick_Config(SystemCoreClock / 1000)) {
    while (1);
  }

  while (1) {

    // on
    GPIOSetBitValue( LED_PORT, LED_BIT, LED_ON );
    delay_ms(DELAY_LEN_MS);

    // off
    GPIOSetBitValue( LED_PORT, LED_BIT, LED_OFF );
    delay_ms(DELAY_LEN_MS);
  }
}
